@@ -7,14 +7,12 @@ use base 'Exporter';
 use English qw(-no_match_vars);
 use List::Util qw(first);
 
-use FusionInventory::Agent::Tools; # runFunction
+use FusionInventory::Agent::Tools;
 use FusionInventory::Agent::Tools::Network;
 
 our @EXPORT = qw(
-    getDeviceBaseInfo
     getDeviceInfo
     getDeviceFullInfo
-    loadModel
 );
 
 my %types = (
@@ -50,6 +48,7 @@ my %sysobjectid_vendors = (
     637   => { vendor => 'Alcatel-Lucent',  type => 'NETWORKING' },
     641   => { vendor => 'Lexmark',         type => 'PRINTER'    },
     674   => { vendor => 'Dell'                                  },
+    714   => { vendor => 'Wyse',            type => 'PRINTER'    },
     1139  => { vendor => 'EMC',             type => 'STORAGE'    },
     1248  => { vendor => 'Epson',           type => 'PRINTER'    },
     1347  => { vendor => 'Kyocera',         type => 'PRINTER'    },
@@ -62,6 +61,7 @@ my %sysobjectid_vendors = (
     2385  => { vendor => 'Sharp',           type => 'PRINTER'    },
     2435  => { vendor => 'Brother',         type => 'PRINTER'    },
     2636  => { vendor => 'Juniper',         type => 'NETWORKING' },
+    2699  => { vendor => 'Axis',            type => 'PRINTER'    },
     3224  => { vendor => 'NetScreen',       type => 'NETWORKING' },
     3977  => { vendor => 'Broadband',       type => 'NETWORKING' },
     5596  => { vendor => 'Tandberg',        type => 'VIDEO'      },
@@ -98,8 +98,8 @@ my %sysdescr_first_word = (
     'emc'            => { vendor => 'EMC',             type => 'STORAGE'    },
     'enterasys'      => { vendor => 'Enterasys',       type => 'NETWORKING' },
     'epson'          => { vendor => 'Epson',           type => 'PRINTER'    },
-    'extreme'        => { vendor => 'Extrem Networks', type => 'NETWORKING' },
-    'extremexos'     => { vendor => 'Extrem Networks', type => 'NETWORKING' },
+    'extreme'        => { vendor => 'Extreme',         type => 'NETWORKING' },
+    'extremexos'     => { vendor => 'Extreme',         type => 'NETWORKING' },
     'foundry'        => { vendor => 'Foundry',         type => 'NETWORKING' },
     'fuji'           => { vendor => 'Fuji',            type => 'NETWORKING' },
     'h3c'            => { vendor => 'H3C',             type => 'NETWORKING' },
@@ -132,257 +132,118 @@ my %sysdescr_first_word = (
 
 my @sysdescr_rules = (
     {
-        match       => qr/^\S+ Service Release/,
-        description => { function => 'FusionInventory::Agent::Tools::Hardware::Alcatel::getDescription' },
-        vendor      => { value    => 'Alcatel' }
+        match => qr/Switch/,
+        type  => 'NETWORKING',
     },
     {
-        match       => qr/AXIS OfficeBasic Network Print Server/,
-        description => { function => 'FusionInventory::Agent::Tools::Hardware::Axis::getDescription' },
-        vendor      => { value    => 'Axis' },
-        type        => { value    => 'PRINTER' }
-    },
-    {
-        match       => qr/^Ethernet Switch$/,
-        description => { oid   => '.1.3.6.1.4.1.674.10895.3000.1.2.100.1.0' },
-        vendor      => { value => 'Dell' },
-        type        => { value => 'NETWORKING' }
-    },
-    {
-        match       => qr/EPSON Built-in/,
-        description => { oid   => '.1.3.6.1.4.1.1248.1.1.3.1.3.8.0' },
-        vendor      => { value => 'Epson' },
-    },
-    {
-        match       => qr/EPSON Internal 10Base-T/,
-        description => { oid   => '.1.3.6.1.2.1.25.3.2.1.3.1' },
-        vendor      => { value => 'Epson' },
-    },
-    {
-        match       => qr/HP ETHERNET MULTI-ENVIRONMENT/,
-        description => { function => 'FusionInventory::Agent::Tools::Hardware::HewlettPackard::getDescription' },
-        vendor      => { value    => 'Hewlett-Packard' }
-    },
-    {
-        match       => qr/A SNMP proxy agent, EEPROM/,
-        description => { function => 'FusionInventory::Agent::Tools::Hardware::HewlettPackard::getDescription' },
-        vendor      => { value    => 'Hewlett-Packard' }
-    },
-    {
-        match       => qr/,HP,JETDIRECT,J/,
-        description => { oid   => '.1.3.6.1.4.1.1229.2.2.2.1.15.1' },
-        vendor      => { value => 'Kyocera' },
-        type        => { value => 'PRINTER' }
-    },
-    {
-        match       => qr/JETDIRECT/,
-        type        => { value => 'PRINTER' }
-    },
-    {
-        match       => qr/^KYOCERA (MITA Printing System|Print I\/F)$/,
-        description => { function => 'FusionInventory::Agent::Tools::Hardware::Kyocera::getDescription' },
-        vendor      => { value    => 'Kyocera' },
-        type        => { value    => 'PRINTER' }
-    },
-    {
-        match       => qr/^SB-110$/,
-        description => { function => 'FusionInventory::Agent::Tools::Hardware::Kyocera::getDescription' },
-        vendor      => { value    => 'Kyocera' },
-        type        => { value    => 'PRINTER' }
-    },
-    {
-        match       => qr/RICOH NETWORK PRINTER/,
-        description => { oid => '.1.3.6.1.4.1.11.2.3.9.1.1.7.0' },
-        vendor      => { value => 'Ricoh' },
-        type        => { value => 'PRINTER' }
-    },
-    {
-        match       => qr/SAMSUNG NETWORK PRINTER,ROM/,
-        description => { oid => '.1.3.6.1.4.1.236.11.5.1.1.1.1.0' },
-        vendor      => { value => 'Samsung' },
-        type        => { value => 'PRINTER' }
-    },
-    {
-        match       => qr/Samsung(.*);S\/N(.*)/,
-        description => { oid => '.1.3.6.1.4.1.236.11.5.1.1.1.1.0' }
-    },
-    {
-        match        => qr/Linux/,
-        description => { function => 'FusionInventory::Agent::Tools::Hardware::Wyse::getDescription' },
-        vendor      => { value    => 'Wyse' },
-    },
-    {
-        match       => qr/ZebraNet PrintServer/,
-        description => { function => 'FusionInventory::Agent::::Tools::Hardware::Zebranet::getDescription' },
-        vendor      => { value    => 'Zebranet' },
-        type        => { value    => 'PRINTER' }
-    },
-    {
-        match       => qr/ZebraNet Wired PS/,
-        description => { function => 'FusionInventory::Agent::Tools::Hardware::Zebranet::getDescription' },
-        vendor      => { value    => 'Zebranet' },
-    },
-    {
-        match       => qr/Nortel Networks$/,
-        vendor      => { value    => 'Nortel' },
-        type        => { value    => 'NETWORKING' }
+        match => qr/JETDIRECT/,
+        type  => 'PRINTER',
     },
 );
 
 # common base variables
 my %base_variables = (
-    MAC          => {
-        mapping => 'macaddr',
-        default => '.1.3.6.1.2.1.17.1.1.0',
-        type    => 'mac',
-    },
     CPU          => {
-        mapping => 'cpu',
-        default => '.1.3.6.1.4.1.9.9.109.1.1.1.1.3.1',
-        type    => 'count',
+        oid  => '.1.3.6.1.4.1.9.9.109.1.1.1.1.3.1',
+        type => 'count',
+    },
+    SNMPHOSTNAME => {
+        oid  => '.1.3.6.1.2.1.1.5.0',
+        type => 'string',
     },
     LOCATION     => {
-        mapping => 'location',
-        default => '.1.3.6.1.2.1.1.6.0',
-        type    => 'string',
+        oid  => '.1.3.6.1.2.1.1.6.0',
+        type => 'string',
     },
     CONTACT      => {
-        mapping => 'contact',
-        default => '.1.3.6.1.2.1.1.4.0',
-        type    => 'string',
-    },
-    COMMENTS     => {
-        mapping => 'comments',
-        default => '.1.3.6.1.2.1.1.1.0',
-        type    => 'string',
+        oid  => '.1.3.6.1.2.1.1.4.0',
+        type => 'string',
     },
     UPTIME       => {
-        mapping => 'uptime',
-        default => '.1.3.6.1.2.1.1.3.0',
-        type    => 'string',
-    },
-    SERIAL       => {
-        mapping => 'serial',
-        default => [
-            '.1.3.6.1.2.1.47.1.1.1.1.11.1',
-            '.1.3.6.1.2.1.47.1.1.1.1.11.2',
-            '.1.3.6.1.2.1.47.1.1.1.1.11.1001',
-        ],
-        type    => 'serial',
-    },
-    NAME         => {
-        mapping => 'name',
-        default => '.1.3.6.1.2.1.1.5.0',
-        type    => 'string',
-    },
-    MANUFACTURER => {
-        mapping => 'enterprise',
-        default => '.1.3.6.1.2.1.43.8.2.1.14.1.1',
-        type    => 'string',
-    },
-    OTHERSERIAL  => {
-        mapping => 'otherserial',
-        type    => 'serial',
+        oid  => '.1.3.6.1.2.1.1.3.0',
+        type => 'string',
     },
     MEMORY       => {
-        mapping => 'memory',
-        default => [
+        oid  => [
             '.1.3.6.1.4.1.9.2.1.8.0',
             '.1.3.6.1.2.1.25.2.3.1.5.1',
         ],
-        type    => 'memory',
+        type => 'memory',
     },
     RAM          => {
-        mapping => 'ram',
-        default => '.1.3.6.1.4.1.9.3.6.6.0',
-        type    => 'memory',
+        oid  => '.1.3.6.1.4.1.9.3.6.6.0',
+        type => 'memory',
     },
 );
 
 # common interface variables
 my %interface_variables = (
     IFNUMBER         => {
-        mapping => 'ifIndex',
-        default => '.1.3.6.1.2.1.2.2.1.1',
-        type    => 'none'
+        oid  => '.1.3.6.1.2.1.2.2.1.1',
+        type => 'none'
     },
     IFDESCR          => {
-        mapping => 'ifdescr',
-        default => '.1.3.6.1.2.1.2.2.1.2',
-        type    => 'string',
+        oid  => '.1.3.6.1.2.1.2.2.1.2',
+        type => 'string',
     },
     IFNAME           => {
-        mapping => 'ifName',
-        default => [
+        oid  => [
             '.1.3.6.1.2.1.31.1.1.1.1',
             '.1.3.6.1.2.1.2.2.1.2',
         ],
-        type    => 'string',
+        type => 'string',
     },
     IFTYPE           => {
-        mapping => 'ifType',
-        default => '.1.3.6.1.2.1.2.2.1.3',
-        type    => 'constant',
+        oid  => '.1.3.6.1.2.1.2.2.1.3',
+        type => 'constant',
     },
     IFMTU            => {
-        mapping => 'ifmtu',
-        default => '.1.3.6.1.2.1.2.2.1.4',
-        type    => 'count',
+        oid  => '.1.3.6.1.2.1.2.2.1.4',
+        type => 'count',
     },
     IFSPEED          => {
-        mapping => 'ifspeed',
-        default => '.1.3.6.1.2.1.2.2.1.5',
-        type    => 'count',
+        oid  => '.1.3.6.1.2.1.2.2.1.5',
+        type => 'count',
     },
     IFSTATUS         => {
-        mapping => 'ifstatus',
-        default => '.1.3.6.1.2.1.2.2.1.8',
-        type    => 'constant',
+        oid  => '.1.3.6.1.2.1.2.2.1.8',
+        type => 'constant',
     },
     IFINTERNALSTATUS => {
-        mapping => 'ifinternalstatus',
-        default => '.1.3.6.1.2.1.2.2.1.7',
-        type    => 'constant',
+        oid  => '.1.3.6.1.2.1.2.2.1.7',
+        type => 'constant',
     },
     IFLASTCHANGE     => {
-        mapping => 'iflastchange',
-        default => '.1.3.6.1.2.1.2.2.1.9',
-        type    => 'none'
+        oid  => '.1.3.6.1.2.1.2.2.1.9',
+        type => 'none'
     },
     IFINOCTETS       => {
-        mapping => 'ifinoctets',
-        default => '.1.3.6.1.2.1.2.2.1.10',
-        type    => 'count',
+        oid  => '.1.3.6.1.2.1.2.2.1.10',
+        type => 'count',
     },
     IFOUTOCTETS      => {
-        mapping => 'ifoutoctets',
-        default => '.1.3.6.1.2.1.2.2.1.16',
-        type    => 'count',
+        oid  => '.1.3.6.1.2.1.2.2.1.16',
+        type => 'count',
     },
     IFINERRORS       => {
-        mapping => 'ifinerrors',
-        default => '.1.3.6.1.2.1.2.2.1.14',
-        type    => 'count',
+        oid  => '.1.3.6.1.2.1.2.2.1.14',
+        type => 'count',
     },
     IFOUTERRORS      => {
-        mapping => 'ifouterrors',
-        default => '.1.3.6.1.2.1.2.2.1.20',
-        type    => 'count',
+        oid  => '.1.3.6.1.2.1.2.2.1.20',
+        type => 'count',
     },
     MAC              => {
-        mapping => 'ifPhysAddress',
-        default => '.1.3.6.1.2.1.2.2.1.6',
-        type    => 'mac',
+        oid  => '.1.3.6.1.2.1.2.2.1.6',
+        type => 'mac',
     },
     IFPORTDUPLEX     => {
-        mapping => 'portDuplex',
-        default => '.1.3.6.1.2.1.10.7.2.1.19',
-        type    => 'constant',
+        oid  => '.1.3.6.1.2.1.10.7.2.1.19',
+        type => 'constant',
     },
     IFALIAS          => {
-        mapping => 'ifAlias',
-        default => '.1.3.6.1.2.1.31.1.1.1.18',
-        type    => 'string',
+        oid  => '.1.3.6.1.2.1.31.1.1.1.18',
+        type => 'string',
     },
 );
 
@@ -453,59 +314,33 @@ my %consumable_variables_from_type = (
     maintenance => 'MAINTENANCEKIT',
 );
 
-my %consumable_variables_from_mappings = (
-    tonerblack            => 'TONERBLACK',
-    tonerblack2           => 'TONERBLACK2',
-    tonercyan             => 'TONERCYAN',
-    tonermagenta          => 'TONERMAGENTA',
-    toneryellow           => 'TONERYELLOW',
-    wastetoner            => 'WASTETONER',
-    cartridgeblack        => 'CARTRIDGEBLACK',
-    cartridgeblackphoto   => 'CARTRIDGEBLACKPHOTO',
-    cartridgecyan         => 'CARTRIDGECYAN',
-    cartridgecyanlight    => 'CARTRIDGECYANLIGHT',
-    cartridgemagenta      => 'CARTRIDGEMAGENTA',
-    cartridgemagentalight => 'CARTRIDGEMAGENTALIGHT',
-    cartridgeyellow       => 'CARTRIDGEYELLOW',
-    maintenancekit        => 'MAINTENANCEKIT',
-    drumblack             => 'DRUMBLACK',
-    drumcyan              => 'DRUMCYAN',
-    drummagenta           => 'DRUMMAGENTA',
-    drumyellow            => 'DRUMYELLOW',
-);
-
 # printer-specific page counter variables
 my %printer_pagecounters_variables = (
     TOTAL      => {
-        mapping => 'pagecountertotalpages',
-        default => '.1.3.6.1.2.1.43.10.2.1.4.1.1'
+        oid   => '.1.3.6.1.2.1.43.10.2.1.4.1.1'
     },
-    BLACK      => { mapping => 'pagecounterblackpages'       },
-    COLOR      => { mapping => 'pagecountercolorpages'       },
-    RECTOVERSO => { mapping => 'pagecounterrectoversopages'  },
-    SCANNED    => { mapping => 'pagecounterscannedpages'     },
-    PRINTTOTAL => { mapping => 'pagecountertotalpages_print' },
-    PRINTBLACK => { mapping => 'pagecounterblackpages_print' },
-    PRINTCOLOR => { mapping => 'pagecountercolorpages_print' },
-    COPYTOTAL  => { mapping => 'pagecountertotalpages_copy'  },
-    COPYBLACK  => { mapping => 'pagecounterblackpages_copy'  },
-    COPYCOLOR  => { mapping => 'pagecountercolorpages_copy'  },
-    FAXTOTAL   => { mapping => 'pagecountertotalpages_fax'   },
+    BLACK      => { },
+    COLOR      => { },
+    RECTOVERSO => { },
+    SCANNED    => { },
+    PRINTTOTAL => { },
+    PRINTBLACK => { },
+    PRINTCOLOR => { },
+    COPYTOTAL  => { },
+    COPYBLACK  => { },
+    COPYCOLOR  => { },
+    FAXTOTAL   => { },
 );
 
-sub getDeviceBaseInfo {
-    my ($snmp, $datadir) = @_;
+sub getDeviceInfo {
+    my (%params) = @_;
 
-    # retrieve sysdescr value, as it is our primary identification key
-    my $sysdescr = $snmp->get('.1.3.6.1.2.1.1.1.0'); # SNMPv2-MIB::sysDescr.0
-
-    # failure eithers means a network or a credential issue
-    return unless $sysdescr;
+    my $snmp    = $params{snmp};
+    my $datadir = $params{datadir};
 
     my %device;
 
-    # first heuristic:
-    # compute manufacturer and type from sysobjectid (SNMPv2-MIB::sysObjectID.0)
+    # manufacturer, type and model identification attempt, using sysObjectID
     my $sysobjectid = $snmp->get('.1.3.6.1.2.1.1.2.0');
     if ($sysobjectid) {
         my $prefix = qr/(?:
@@ -530,39 +365,83 @@ sub getDeviceBaseInfo {
         }
     }
 
-    # second heuristic:
-    # compute manufacturer and type from first sysdescr word
-    my ($first_word) = $sysdescr =~ /^(\S+)/;
-    my $result = $sysdescr_first_word{lc($first_word)};
+    # manufacturer and type identification attempt, using sysDescr
+    my $sysdescr = $snmp->get('.1.3.6.1.2.1.1.1.0');
+    if ($sysdescr) {
 
-    if ($result) {
-        $device{MANUFACTURER} = $result->{vendor};
-        $device{TYPE}         = $result->{type} if $result->{type};
-    }
+        # first word
+        my ($first_word) = $sysdescr =~ /^(\S+)/;
+        my $result = $sysdescr_first_word{lc($first_word)};
 
-    # third heuristic:
-    # compute manufacturer, type and a more specific identification key
-    # from a list of rules matched against sysdescr value
-    # the first matching rule wins
-    if ($snmp) {
+        if ($result) {
+            $device{MANUFACTURER} = $result->{vendor} if $result->{vendor};
+            $device{TYPE}         = $result->{type} if $result->{type};
+        }
+
+        # whole sysdescr value
         foreach my $rule (@sysdescr_rules) {
             next unless $sysdescr =~ $rule->{match};
-            $device{MANUFACTURER} = _apply_rule($rule->{vendor}, $snmp)
-                if $rule->{vendor};
-            $device{TYPE}         = _apply_rule($rule->{type}, $snmp)
-                if $rule->{type};
-            $device{DESCRIPTION}  = _apply_rule($rule->{description}, $snmp)
-                if $rule->{description};
+            $device{MANUFACTURER} = $rule->{vendor} if $rule->{vendor};
+            $device{TYPE}         = $rule->{type}   if $rule->{type};
             last;
         }
+        $device{DESCRIPTION} = $sysdescr;
     }
 
-    # use sysdescr as default identification key
-    $device{DESCRIPTION}  = $sysdescr if !$device{DESCRIPTION};
+    # fallback model identification attempt, using type-specific OID
+    if (!exists $device{MODEL} && exists $device{TYPE}) {
+        my $type = $device{TYPE};
+        my $model =
+            $type eq 'PRINTER'    ? $snmp->get('.1.3.6.1.2.1.25.3.2.1.3.1')    :
+            $type eq 'NETWORKING' ? $snmp->get('.1.3.6.1.2.1.47.1.1.1.1.13.1') :
+                                    undef;
+        $device{MODEL} = $model if $model;
+    }
 
-    # SNMPv2-MIB::sysName.0
-    my $hostname = $snmp->get('.1.3.6.1.2.1.1.5.0');
-    $device{SNMPHOSTNAME} = $hostname if $hostname;
+    # fallback manufacturer identification attempt, using type-agnostic OID
+    if (!exists $device{MANUFACTURER}) {
+        my $manufacturer = $snmp->get('.1.3.6.1.2.1.43.8.2.1.14.1.1');
+        $device{MANUFACTURER} = $manufacturer if $manufacturer;
+    }
+
+    # remaining informations
+    foreach my $key (keys %base_variables) {
+        my $variable = $base_variables{$key};
+
+        my $raw_value;
+        if (ref $variable->{oid} eq 'ARRAY') {
+            foreach my $oid (@{$variable->{oid}}) {
+                $raw_value = $snmp->get($oid);
+                last if defined $raw_value;
+            }
+        } else {
+            $raw_value = $snmp->get($variable->{oid});
+        }
+        next unless defined $raw_value;
+
+        my $type = $variable->{type};
+        my $value =
+            $type eq 'memory' ? _getCanonicalMemory($raw_value) :
+            $type eq 'string' ? _getCanonicalString($raw_value) :
+            $type eq 'count'  ? _getCanonicalCount($raw_value)  :
+                                $raw_value;
+
+        $device{$key} = $value if defined $value;
+    }
+
+    my $mac = _getMacAddress($snmp);
+    $device{MAC} = $mac if $mac;
+
+    my $serial = _getSerial($snmp, $device{TYPE});
+    $device{SERIAL} = $serial if $serial;
+
+    my $firmware = _getFirmware($snmp, $device{TYPE});
+    $device{FIRMWARE} = $firmware if $firmware;
+
+    my $results = $snmp->walk('.1.3.6.1.2.1.4.20.1.1');
+    $device{IPS}->{IP} =  [
+        sort values %{$results}
+    ] if $results;
 
     return %device;
 }
@@ -603,127 +482,113 @@ sub _loadDeviceModels {
 }
 
 sub _getSerial {
-    my ($snmp, $model) = @_;
+    my ($snmp, $type) = @_;
 
-    return unless $model->{SERIAL};
-    return _getCanonicalSerialNumber($snmp->get($model->{SERIAL}));
+    my @network_oids = (
+        '.1.3.6.1.2.1.47.1.1.1.1.11.1',    # Entity-MIB::entPhysicalSerialNum
+        '.1.3.6.1.2.1.47.1.1.1.1.11.2',    # Entity-MIB::entPhysicalSerialNum
+        '.1.3.6.1.2.1.47.1.1.1.1.11.1001', # Entity-MIB::entPhysicalSerialNum
+        '.1.3.6.1.4.1.2636.3.1.3.0',       # Juniper-MIB
+    );
+
+    my @printer_oids = (
+        '.1.3.6.1.2.1.43.5.1.1.17.1',      # Printer-MIB::prtGeneralSerialNumber
+        '.1.3.6.1.4.1.253.8.53.3.2.1.3.1',       # Xerox-MIB
+        '.1.3.6.1.4.1.367.3.2.1.2.1.4.0',        # Ricoh-MIB
+        '.1.3.6.1.4.1.641.2.1.2.1.6.1',          # Lexmark-MIB
+        '.1.3.6.1.4.1.1602.1.2.1.4.0',           # Canon-MIB
+        '.1.3.6.1.4.1.2435.2.3.9.4.2.1.5.5.1.0', # Brother-MIB
+    );
+
+    my @oids =
+        $type && $type eq 'NETWORKING' ? @network_oids :
+        $type && $type eq 'PRINTER'    ? @printer_oids :
+                                         (@network_oids, @printer_oids);
+
+    foreach my $oid (@oids) {
+        my $value = $snmp->get($oid);
+        next unless $value;
+        return _getCanonicalSerialNumber($value);
+    }
+
+    return;
+}
+
+sub _getFirmware {
+    my ($snmp, $type) = @_;
+
+    my @oids = (
+        '.1.3.6.1.2.1.47.1.1.1.1.9.1',
+        '.1.3.6.1.2.1.47.1.1.1.1.9.1000',
+        '.1.3.6.1.2.1.47.1.1.1.1.9.1001',
+        '.1.3.6.1.2.1.47.1.1.1.1.10.1',
+        '.1.3.6.1.4.1.9.9.25.1.1.1.2.5'
+    );
+
+    foreach my $oid (@oids) {
+        my $value = $snmp->get($oid);
+        next unless $value;
+        return $value;
+    }
+
+    return;
 }
 
 sub _getMacAddress {
-    my ($snmp, $model) = @_;
+    my ($snmp) = @_;
 
-    my $mac_oid =
-        $model->{MAC} ||
-        ".1.3.6.1.2.1.17.1.1.0"; # SNMPv2-SMI::mib-2.17.1.1.0
-    my $dynmac_oid =
-        $model->{DYNMAC} ||
-        ".1.3.6.1.2.1.2.2.1.6";  # IF-MIB::ifPhysAddress
+    # use BRIDGE-MIB::dot1dBaseBridgeAddress if available
+    my $address_oid = ".1.3.6.1.2.1.17.1.1.0";
+    my $address = _getCanonicalMacAddress($snmp->get($address_oid));
 
-    my $address = _getCanonicalMacAddress($snmp->get($mac_oid));
+    return $address if $address && $address =~ /^$mac_address_pattern$/;
 
-    if (!$address || $address !~ /^$mac_address_pattern$/) {
-        my $macs = $snmp->walk($dynmac_oid);
-        $address =
-            first { $_ ne '00:00:00:00:00:00' }
-            map   { _getCanonicalMacAddress($_) }
-            grep  { $_ }
-            sort  { $a cmp $b }
-            values %{$macs};
-    }
+    # fallback on ports addresses (IF-MIB::ifPhysAddress) if unique
+    my $addresses_oid = ".1.3.6.1.2.1.2.2.1.6";
+    my $addresses = $snmp->walk($addresses_oid);
+    my @addresses =
+        uniq
+        grep { $_ ne '00:00:00:00:00:00' }
+        grep { $_ }
+        map  { _getCanonicalMacAddress($_) }
+        values %{$addresses};
 
-    return $address;
-}
+    return $addresses[0] if @addresses && @addresses == 1;
 
-sub getDeviceInfo {
-    my (%params) = @_;
-
-    my $snmp       = $params{snmp};
-    my $dictionary = $params{dictionary};
-
-    # the device is initialized with basic information
-    # deduced from its sysdescr
-    my %device = getDeviceBaseInfo($snmp, $params{datadir});
-    return unless %device;
-
-    # then, we try to get a matching model from the dictionary,
-    # using its current description as identification key
-    my $model = $dictionary ?
-        $dictionary->getModel($device{DESCRIPTION}) : undef;
-
-    if ($model) {
-        # if found, we complete the device with model-defined mappings
-        $device{MANUFACTURER} = $model->{MANUFACTURER}
-            if $model->{MANUFACTURER};
-        $device{TYPE}         =
-            $model->{TYPE} == 1 ? 'COMPUTER'   :
-            $model->{TYPE} == 2 ? 'NETWORKING' :
-            $model->{TYPE} == 3 ? 'PRINTER'    :
-                                  undef
-            if $model->{TYPE};
-
-        $device{MAC}       = _getMacAddress($snmp, $model);
-        $device{SERIAL}    = _getSerial($snmp, $model);
-        $device{MODELSNMP} = $model->{MODELSNMP};
-        $device{FIRMWARE}  = $model->{FIRMWARE};
-    } else {
-        # otherwise, we fallback on default mappings
-        $device{MAC} = _getMacAddress($snmp);
-    }
-
-    return %device;
-}
-
-sub _apply_rule {
-    my ($rule, $snmp) = @_;
-
-    return unless $rule;
-
-    if ($rule->{value}) {
-        return $rule->{value};
-    }
-
-    if ($rule->{oid}) {
-        return $snmp->get($rule->{oid});
-    }
-
-    if ($rule->{function}) {
-        my ($module, $function) = $rule->{function} =~ /^(\S+)::(\S+)$/;
-        return runFunction(
-            module   => $module,
-            function => $function,
-            params   => $snmp,
-            load     => 1
-        );
-    }
+    return;
 }
 
 sub getDeviceFullInfo {
     my (%params) = @_;
 
     my $snmp   = $params{snmp};
-    my $model  = $params{model};
     my $logger = $params{logger};
 
     # first, let's retrieve basic device informations
-    my %info = getDeviceBaseInfo($snmp, $params{datadir});
+    my %info = getDeviceInfo(%params);
     return unless %info;
 
-    # unfortunatly, some elements differs between discovery
-    # and inventory response
-    delete $info{DESCRIPTION};
-    delete $info{SNMPHOSTNAME};
+    # description is defined as DESCRIPTION for discovery
+    # and COMMENTS for inventory
+    if (exists $info{DESCRIPTION}) {
+        $info{COMMENTS} = $info{DESCRIPTION};
+        delete $info{DESCRIPTION};
+    }
+
+    # host name is defined as SNMPHOSTNAME for discovery
+    # and NAME for inventory
+    if (exists $info{SNMPHOSTNAME}) {
+        $info{NAME} = $info{SNMPHOSTNAME};
+        delete $info{SNMPHOSTNAME};
+    }
 
     # device ID is set from the server request
     $info{ID} = $params{id};
 
     # device TYPE is set either:
     # - from the server request,
-    # - from the model type
     # - from initial identification
-    $info{TYPE} =
-            $params{type}            ? $params{type}          :
-            $model && $model->{TYPE} ? $types{$model->{TYPE}} :
-                                       $info{TYPE}            ;
+    $info{TYPE} = $params{type} || $info{TYPE};
 
     # second, use results to build the object
     my $device = { INFO => \%info };
@@ -731,22 +596,19 @@ sub getDeviceFullInfo {
     _setGenericProperties(
         device => $device,
         snmp   => $snmp,
-        model  => $model,
         logger => $logger
     );
 
     _setPrinterProperties(
-        device  => $device,
+        device => $device,
         snmp   => $snmp,
-        model  => $model,
         logger => $logger
     ) if $info{TYPE} && $info{TYPE} eq 'PRINTER';
 
     _setNetworkingProperties(
-        device      => $device,
-        snmp        => $snmp,
-        model       => $model,
-        logger      => $logger
+        device  => $device,
+        snmp    => $snmp,
+        logger  => $logger
     ) if $info{TYPE} && $info{TYPE} eq 'NETWORKING';
 
     # convert ports hashref to an arrayref, sorted by interface number
@@ -769,98 +631,25 @@ sub _setGenericProperties {
 
     my $device = $params{device};
     my $snmp   = $params{snmp};
-    my $model  = $params{model};
     my $logger = $params{logger};
 
-    my $firmware;
-    if ($model->{oids}->{firmware}) {
-        $firmware = $snmp->get($model->{oids}->{firmware});
-    } else {
-        my @parts;
-        if ($model->{oids}->{firmware1}) {
-            my $firmware1 = $snmp->get($model->{oids}->{firmware1});
-            push @parts, $firmware1 if $firmware1;
-        }
-        if ($model->{oids}->{firmware2}) {
-            my $firmware2 = $snmp->get($model->{oids}->{firmware2});
-            push @parts, $firmware2 if $firmware2;
-        }
-        $firmware = join(' ', @parts) if @parts;
-    }
-    $device->{INFO}->{FIRMWARE} = $firmware if $firmware;
 
-    foreach my $key (keys %base_variables) {
-        # don't overwrite known values
-        next if $device->{INFO}->{$key};
-
-        my $variable = $base_variables{$key};
-        my $raw_value;
-
-        # first, try model mapping, if defined
-        if ($model->{oids}->{$variable->{mapping}}) {
-            $raw_value = $snmp->get($model->{oids}->{$variable->{mapping}});
-        }
-
-        # second, try default value, if defined
-        if (!defined $raw_value) {
-            if ($variable->{default}) {
-                if (ref $variable->{default} eq 'ARRAY') {
-                    foreach my $default (@{$variable->{default}}) {
-                        $raw_value = $snmp->get($default);
-                        last if defined $raw_value;
-                    }
-                } else {
-                    $raw_value = $snmp->get($variable->{default});
-                }
-            }
-        }
-
-        next unless defined $raw_value;
-
-        my $type = $variable->{type};
-        my $value =
-            $type eq 'mac'    ? _getCanonicalMacAddress($raw_value)   :
-            $type eq 'memory' ? _getCanonicalMemory($raw_value)       :
-            $type eq 'serial' ? _getCanonicalSerialNumber($raw_value) :
-            $type eq 'string' ? _getCanonicalString($raw_value)       :
-            $type eq 'count'  ? _getCanonicalCount($raw_value)        :
-                                $raw_value;
-
-        $device->{INFO}->{$key} = $value if defined $value;
-    }
-
-    my $results = $snmp->walk(
-        $model->{oids}->{ipAdEntAddr} || '.1.3.6.1.2.1.4.20.1.1'
-    );
-    $device->{INFO}->{IPS}->{IP} =  [
-        sort values %{$results}
-    ] if $results;
 
     # ports is a sparse list of network ports, indexed by native port number
     my $ports;
 
     foreach my $key (keys %interface_variables) {
         my $variable = $interface_variables{$key};
+        next unless $variable->{oid};
 
         my $results;
-
-        # first, try model mapping, if defined
-        if ($model->{oids}->{$variable->{mapping}}) {
-            $results = $snmp->walk($model->{oids}->{$variable->{mapping}});
-        }
-
-        # second, try default value, if defined
-        if (!$results) {
-            if ($variable->{default}) {
-                if (ref $variable->{default} eq 'ARRAY') {
-                    foreach my $default (@{$variable->{default}}) {
-                        $results = $snmp->walk($default);
-                        last if $results;
-                    }
-                } else {
-                    $results = $snmp->walk($variable->{default});
-                }
+        if (ref $variable->{oid} eq 'ARRAY') {
+            foreach my $oid (@{$variable->{oid}}) {
+                $results = $snmp->walk($oid);
+                last if $results;
             }
+        } else {
+            $results = $snmp->walk($variable->{oid});
         }
         next unless $results;
 
@@ -878,9 +667,7 @@ sub _setGenericProperties {
         }
     }
 
-    $results = $snmp->walk(
-        $model->{oids}->{ifaddr} || '.1.3.6.1.2.1.4.20.1.2'
-    );
+    my $results = $snmp->walk('.1.3.6.1.2.1.4.20.1.2');
     # each result matches the following scheme:
     # $prefix.$i.$j.$k.$l = $value
     # with $i.$j.$k.$l as IP address, and $value as port id
@@ -907,26 +694,9 @@ sub _setPrinterProperties {
 
     my $device = $params{device};
     my $snmp   = $params{snmp};
-    my $model  = $params{model};
     my $logger = $params{logger};
 
-    if (!$device->{INFO}->{MODEL}) {
-        $device->{INFO}->{MODEL} = $snmp->get(
-            $model->{oids}->{model} || '.1.3.6.1.2.1.25.3.2.1.3.1'
-        );
-    }
-
     # consumable levels
-
-    # index model-provided mappings
-    my %consumable_variables_from_oids;
-    foreach my $key (sort keys %consumable_variables_from_mappings) {
-        next unless $model->{oids}->{$key};
-        my $variable = $consumable_variables_from_mappings{$key};
-        $consumable_variables_from_oids{$model->{oids}->{$key}} = $variable;
-    }
-
-    # enumerate consumables
     foreach my $index (1 .. 10) {
         my $description_oid = '.1.3.6.1.2.1.43.11.1.1.6.1.' . $index;
         my $description = hex2char($snmp->get($description_oid));
@@ -944,7 +714,6 @@ sub _setPrinterProperties {
 
         # consumable identification
         my $variable =
-            $consumable_variables_from_oids{$description_oid} ||
             _getConsumableVariableFromDescription($description);
 
         next unless $variable;
@@ -955,8 +724,7 @@ sub _setPrinterProperties {
     # page counters
     foreach my $key (keys %printer_pagecounters_variables) {
         my $variable = $printer_pagecounters_variables{$key};
-        my $oid = $model->{oids}->{$variable->{mapping}} ||
-                  $variable->{default};
+        my $oid = $variable->{oid};
         my $value = $snmp->get($oid);
         next unless defined $value;
         if (!_isInteger($value)) {
@@ -1001,23 +769,14 @@ sub _setNetworkingProperties {
 
     my $device = $params{device};
     my $snmp   = $params{snmp};
-    my $model  = $params{model};
     my $logger = $params{logger};
 
-    if (!$device->{INFO}->{MODEL}) {
-        $device->{INFO}->{MODEL} = $snmp->get(
-            $model->{oids}->{entPhysicalModelName} || '.1.3.6.1.2.1.47.1.1.1.1.13.1'
-        )
-    }
-
     my $ports    = $device->{PORTS}->{PORT};
 
-    my $vlans = $snmp->walk($model->{oids}->{vtpVlanName} || '.1.3.6.1.4.1.9.9.46.1.3.1.1.4.1');
+    my $vlans = $snmp->walk('.1.3.6.1.4.1.9.9.46.1.3.1.1.4.1');
 
     # Detect VLAN
-    my $results = $snmp->walk(
-        $model->{oids}->{vmvlan} || '.1.3.6.1.4.1.9.9.68.1.2.2.1.2'
-    );
+    my $results = $snmp->walk('.1.3.6.1.4.1.9.9.68.1.2.2.1.2');
     # each result matches either of the following schemes:
     # $prefix.$i.$j = $value, with $j as port id, and $value as vlan id
     # $prefix.$i    = $value, with $i as port id, and $value as vlan id
@@ -1041,14 +800,12 @@ sub _setNetworkingProperties {
 
     _setTrunkPorts(
         snmp   => $snmp,
-        model  => $model,
         ports  => $ports,
         logger => $logger
     );
 
     _setConnectedDevicesInfo(
         snmp   => $snmp,
-        model  => $model,
         ports  => $ports,
         logger => $logger,
         vendor => $device->{INFO}->{MANUFACTURER}
@@ -1056,7 +813,6 @@ sub _setNetworkingProperties {
 
     _setAssociatedMacAddresses(
         snmp         => $snmp,
-        model        => $model,
         ports        => $ports,
         logger       => $logger,
     );
@@ -1078,25 +834,6 @@ sub _isInteger {
     $_[0] =~ /^[+-]?\d+$/;
 }
 
-
-sub loadModel {
-    my ($file) = @_;
-
-    my $model = XML::TreePP->new()->parsefile($file)->{model};
-
-    my %oids =
-        map  { $_->{mapping_name} => $_->{oid} }
-        grep { $_->{mapping_name} }
-        @{$model->{oidlist}->{oidobject}};
-
-    return {
-        ID   => 1,
-        NAME => $model->{name},
-        TYPE => $model->{type},
-        oids => \%oids
-    }
-}
-
 sub _getCanonicalMacAddress {
     my ($value) = @_;
 
@@ -1368,24 +1105,13 @@ sub _getConnectedDevicesInfoCDP {
     my (%params) = @_;
 
     my $snmp   = $params{snmp};
-    my $model  = $params{model};
 
     my $results;
-    my $cdpCacheAddress    = $snmp->walk(
-        $model->{oids}->{cdpCacheAddress}    || '.1.3.6.1.4.1.9.9.23.1.2.1.1.4'
-    );
-    my $cdpCacheVersion    = $snmp->walk(
-        $model->{oids}->{cdpCacheVersion}    || '.1.3.6.1.4.1.9.9.23.1.2.1.1.5'
-    );
-    my $cdpCacheDeviceId   = $snmp->walk(
-        $model->{oids}->{cdpCacheDeviceId}   || '.1.3.6.1.4.1.9.9.23.1.2.1.1.6'
-    );
-    my $cdpCacheDevicePort = $snmp->walk(
-        $model->{oids}->{cdpCacheDevicePort} || '.1.3.6.1.4.1.9.9.23.1.2.1.1.7'
-    );
-    my $cdpCachePlatform   = $snmp->walk(
-        $model->{oids}->{cdpCachePlatform}   || '.1.3.6.1.4.1.9.9.23.1.2.1.1.8'
-    );
+    my $cdpCacheAddress    = $snmp->walk('.1.3.6.1.4.1.9.9.23.1.2.1.1.4');
+    my $cdpCacheVersion    = $snmp->walk('.1.3.6.1.4.1.9.9.23.1.2.1.1.5');
+    my $cdpCacheDeviceId   = $snmp->walk('.1.3.6.1.4.1.9.9.23.1.2.1.1.6');
+    my $cdpCacheDevicePort = $snmp->walk('.1.3.6.1.4.1.9.9.23.1.2.1.1.7');
+    my $cdpCachePlatform   = $snmp->walk('.1.3.6.1.4.1.9.9.23.1.2.1.1.8');
 
     # each cdp variable matches the following scheme:
     # $prefix.x.y = $value
@@ -1420,24 +1146,13 @@ sub _getConnectedDevicesInfoLLDP {
     my (%params) = @_;
 
     my $snmp   = $params{snmp};
-    my $model  = $params{model};
 
     my $results;
-    my $lldpRemChassisId = $snmp->walk(
-        $model->{oids}->{lldpRemChassisId} || '.1.0.8802.1.1.2.1.4.1.1.5'
-    );
-    my $lldpRemPortId    = $snmp->walk(
-        $model->{oids}->{lldpRemPortId}    || '.1.0.8802.1.1.2.1.4.1.1.7'
-    );
-    my $lldpRemPortDesc  = $snmp->walk(
-        $model->{oids}->{lldpRemPortDesc}  || '.1.0.8802.1.1.2.1.4.1.1.8'
-    );
-    my $lldpRemSysName   = $snmp->walk(
-        $model->{oids}->{lldpRemSysName}   || '.1.0.8802.1.1.2.1.4.1.1.9'
-    );
-    my $lldpRemSysDesc   = $snmp->walk(
-        $model->{oids}->{lldpRemSysDesc}   || '.1.0.8802.1.1.2.1.4.1.1.10'
-    );
+    my $lldpRemChassisId = $snmp->walk('.1.0.8802.1.1.2.1.4.1.1.5');
+    my $lldpRemPortId    = $snmp->walk('.1.0.8802.1.1.2.1.4.1.1.7');
+    my $lldpRemPortDesc  = $snmp->walk('.1.0.8802.1.1.2.1.4.1.1.8');
+    my $lldpRemSysName   = $snmp->walk('.1.0.8802.1.1.2.1.4.1.1.9');
+    my $lldpRemSysDesc   = $snmp->walk('.1.0.8802.1.1.2.1.4.1.1.10');
 
     # dot1dBasePortIfIndex
     my $port2interface = $snmp->walk('.1.3.6.1.2.1.17.1.4.1.2');
@@ -1448,8 +1163,10 @@ sub _getConnectedDevicesInfoLLDP {
 
     while (my ($suffix, $mac) = each %{$lldpRemChassisId}) {
         my $id           = _getElement($suffix, -2);
-        my $interface_id = $params{vendor} eq 'Juniper' ?
-            $id : $port2interface->{$id};
+        my $interface_id =
+            ! exists $port2interface->{$id} ? $id                   :
+            $params{vendor} eq 'Juniper'    ? $id                   :
+                                              $port2interface->{$id};
         $results->{$interface_id} = {
             SYSMAC   => lc(alt2canonical($mac)),
             IFDESCR  => $lldpRemPortDesc->{$suffix},
@@ -1467,7 +1184,6 @@ sub _setTrunkPorts {
 
     my $trunk_ports = _getTrunkPorts(
         snmp  => $params{snmp},
-        model => $params{model}
     );
     return unless $trunk_ports;
 
@@ -1489,13 +1205,9 @@ sub _getTrunkPorts {
     my (%params) = @_;
 
     my $snmp   = $params{snmp};
-    my $model  = $params{model};
 
     my $results;
-    my $vlanStatus = $snmp->walk(
-        $model->{oids}->{vlanTrunkPortDynamicStatus} ||
-        '.1.3.6.1.4.1.9.9.46.1.6.1.1.14'
-    );
+    my $vlanStatus = $snmp->walk('.1.3.6.1.4.1.9.9.46.1.6.1.1.14');
 
     # Values:
     # 1 : trunking
@@ -1521,23 +1233,10 @@ This module provides some hardware-related functions.
 
 =head1 FUNCTIONS
 
-=head2 getDeviceBaseInfo($snmp)
-
-return a minimal set of information for a device through SNMP, according to a
-set of rules hardcoded in the agent.
-
 =head2 getDeviceInfo(%params)
 
-return a limited set of information for a device through SNMP, according to a
-set of rules hardcoded in the agent and the usage of generic knowledge base,
-the dictionary.
+return a limited set of information for a device through SNMP.
 
 =head2 getDeviceFullInfo(%params)
 
-return a full set of information for a device through SNMP, according to a
-set of rules hardcoded in the agent and the usage of a device-specific set of mappings, the model.
-
-=head2 loadModel($file)
-
-Load an SNMP description model from given file.
-package FusionInventory::Agent::Tools::Hardware::Generic;
+return a full set of information for a device through SNMP.
